from click import secho as echo


COLOR_MAP = {
    'error': 'red',
    'fatal': 'red',
    'warning': 'yellow'
}


def secho(**kwargs):
    lvl = kwargs.get('lvl', 'debug')
    msg = kwargs.get('msg', '')
    loader = kwargs.get('loader', 'Default')

    if msg:
        echo(
            '[{lvl}][{loader}]\t{msg}'.format(
                lvl=lvl.upper(), loader=loader.upper(), msg=msg
            ),
            fg=COLOR_MAP.get(lvl.lower(), 'cyan'),
            err=True  # workaround to print but not to env var output
        )



EXTS = ['.env', '.yml', '.yaml', '.json', '.js']


def get_defaults():
    return [ext if ext == '.env' else 'env{}'.format(ext) for ext in EXTS]


def is_yaml(ext):
    return ext.lower() in ('.yml', '.yaml')


def is_json(ext):
    return ext.lower() in ('.json', '.js')



try:
    from simplejson import dumps
except ImportError:
    from json import dumps

import click

from .loader import (
    load_default,
    load_file,
    load_system,
    load_api,
)


@click.command()
@click.version_option()
@click.option(
    '--api',
    is_flag=False,
    help='Endpoint for API sourced environment varialbes.')
@click.option(
    '-t',
    '--token',
    is_flag=False,
    help='Token for API sourced environment variables before others.')
@click.option(
    '--no_default',
    is_flag=True,
    help='Do not load from default local file(s).')
@click.option(
    '-s',
    '--system',
    is_flag=True,
    help='Load system environment variables before local ones.')
@click.option(
    '-v',
    '--verbose',
    is_flag=True,
    help='Enables verbose mode.')
@click.option(
    '-c',
    '--csv',
    is_flag=True,
    help='Comma instead of space separated KEY=VALUE pairs.')
@click.option(
    '-j',
    '--json',
    is_flag=True,
    help='Output in JSON string.')
@click.argument(
    'files',
    nargs=-1,
    type=click.Path(exists=True, dir_okay=False, resolve_path=True))
def cli(api, token, no_default, system, verbose, csv, json, files):
    delimiter = ',' if csv else ' '
    env_vars = {}

    if api and token:
        env_vars.update(load_api(api, token, verbose))

    if system:
        env_vars.update(load_system(verbose))

    if not len(files):
        if not no_default:
            env_vars.update(load_default(verbose))
    else:
        for fname in files:
            env_vars.update(load_file(fname, verbose))

    if env_vars:
        if json:
            return click.echo(dumps(env_vars))

        return click.echo(delimiter.join([
            '{0}={1}'.format(k, v) for k, v in env_vars.items()
        ]))



from print_env.exts import (
    EXTS,
    get_defaults,
    is_yaml,
    is_json,
)


def test_get_defaults():
    defaults = get_defaults()
    for f in defaults:
        assert f.endswith(tuple(EXTS))


def test_is_yaml():
    cases = {
        '.yml': True,
        '.yaml': True,
        '.json': False,
        '.js': False,
        '.env': False
    }
    for k, v in cases.items():
        assert is_yaml(k) is v


def test_is_json():
    cases = {
        '.yml': False,
        '.yaml': False,
        '.json': True,
        '.js': True,
        '.env': False
    }
    for k, v in cases.items():
        assert is_json(k) is v



import os
import codecs

# dotenv loader
from dotenv import dotenv_values
# yaml loader
import yaml
try:
    from yaml import CLoader as YLoader
except ImportError:
    from yaml import Loader as YLoader
# json loader
try:
    import simplejson as json
except ImportError:
    import json
# API loader - requests
import requests
# GnuPG for decrypting encrypted API loaded env vars
import gnupg

from .exts import (
    get_defaults,
    is_yaml,
    is_json,
)
from .utils import secho


def load_default(verbose=False):
    cwd = os.getcwd()
    defaults = get_defaults()

    for fname in defaults:
        env_file = os.path.join(cwd, fname)

        if os.path.isfile(env_file):
            return load_file(env_file, verbose)
    else:
        if verbose:
            secho(
                msg='No default ({ds}) found at {cwd}/'.format(
                    ds=', '.join(defaults),
                    cwd=cwd
                ).strip(),
                lvl='warning'
            )

        return {}


def load_file(fname, verbose=False):
    env_vars = {}

    try:
        _, ext = os.path.splitext(fname)

        if not is_yaml(ext) and not is_json(ext):
            env_vars = dotenv_values(fname)
        else:
            with codecs.open(fname) as f:
                if is_yaml(ext):
                    env_vars = yaml.load(f, Loader=YLoader)
                elif is_json(ext):
                    env_vars = json.load(f)

        if verbose:
            if not env_vars:
                secho(
                    msg='Invalid file {}'.format(fname),
                    lvl='warning',
                    loader='file'
                )
            else:
                secho(
                    msg='From file {}'.format(fname),
                    lvl='debug',
                    loader='file'
                )
    except Exception as e:
        if verbose:
            secho(
                msg='Failed to load {f}\n{e}'.format(
                    f=fname,
                    e=e
                ),
                lvl='error',
                loader='file'
            )

    return env_vars


def load_system(verbose=False):
    if verbose:
        secho(
            msg='From system environment variables',
            lvl='debug',
            loader='system'
        )

    return dict(os.environ)


def load_api(api, token, verbose=False):
    env_vars = {}

    try:
        jwt, project_id, stage = token.split('/')
    except ValueError:
        if verbose:
            secho(msg='Invalid token', lvl='error', loader='API')

        return env_vars

    params = {'project_id': project_id, 'stage': stage, 'encrypt': 0}
    r = requests.get(api, params=params, headers={'portunus-jwt': jwt})

    if r.status_code != 200:
        if verbose:
            try:
                err = r.json()
                err = err.get('message', err)
            except Exception:
                err = r.text

            secho(msg='API error - {}'.format(err), lvl='error', loader='API')

        return env_vars

    try:
        data = r.json()
        encrypted = data.get('encrypted', False)
        env_vars = data.get('vars', {})

        if encrypted:
            gpg = gnupg.GPG()
            env_vars = json.loads(str(gpg.decrypt(env_vars)))

        if verbose:
            if not env_vars:
                secho(
                    msg='No vars loaded from API {}'.format(api),
                    lvl='warning',
                    loader='API'
                )
            else:
                secho(
                    msg='From API {} (Encrypted: {})'.format(api, encrypted),
                    lvl='debug',
                    loader='API'
                )
    except Exception as e:
        if verbose:
            secho(msg='API error - {}'.format(e), lvl='error', loader='API')

        env_vars = {}

    return env_vars



import os
import sys
import codecs
from setuptools import setup, find_packages
from shutil import rmtree

here = os.path.abspath(os.path.dirname(__file__))
package = 'print_env'

with codecs.open(os.path.join(here, 'README.md')) as f:
    long_description = '\n' + f.read()

version = {}
with codecs.open(os.path.join(here, package, 'version.py')) as f:
    exec(f.read(), version)


def print_run(cmd, err_exit=False):
    print('RUNNING: {}'.format(cmd))
    r = os.system(cmd)
    if err_exit and r != 0:
        sys.exit(r)


if sys.argv[-1] == 'publish':
    try:
        rmtree(os.path.join(here, 'dist'))
    except FileNotFoundError:
        pass

    print_run('{0} setup.py sdist bdist_wheel'.format(sys.executable), True)
    print_run('env $(print-env twine.env) twine upload dist/*', True)
    print_run('git tag v{}'.format(version['__version__']))
    print_run('git push --tags')
    sys.exit()

required = [
    'python-dotenv>=0.8.2',
    'pyyaml>=3.12',
    'click>=6.7',
    'requests>=2.0',
    'python-gnupg>=0.4',
]

setup(
    name='print-env',
    version=version['__version__'],
    description='CLI to print environment variables from supported files.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Runzhou Li (Leo)',
    author_email='me@runzhou.li',
    url='https://github.com/woozyking/print-env',
    packages=find_packages(exclude=['tests']),
    install_requires=required,
    entry_points={
        'console_scripts': [
            'print-env={}.cli:cli'.format(package)
        ]
    },
    package_data={
        '': ['LICENSE']
    },
    license='MIT',
    python_requires='>=3.6',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ]
)



import os
import shutil
import json

from print_env.loader import (
    load_default,
    load_file,
    load_system,
)
from print_env.exts import (
    EXTS,
    get_defaults,
)

# patch to make FileNotFoundError "work" on Python 2.7
try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError


TEST_PRE = 'print_env-test'
TEMP_PRE = 'test_'
CWD = os.getcwd()


# move default files temporarily for testing no file case
def _move(back=False):
    for f in get_defaults():
        env = os.path.join(CWD, f)
        temp = os.path.join(CWD, '{0}{1}'.format(TEMP_PRE, f))
        try:
            if back:
                shutil.move(temp, env)
            else:
                shutil.move(env, temp)
        except FileNotFoundError:
            pass


def test_default_nofile():
    _move()
    assert load_default() == {}
    _move(back=True)


def test_default_files():
    _move()
    files = get_defaults()
    for ef in files:
        env_file, key, val = _write_file(ef)
        assert load_default() == {key: val}
        os.remove(env_file)
    _move(back=True)


def test_files():
    _move()
    files = ['{0}{1}'.format(TEST_PRE, ext) for ext in EXTS]
    for ef in files:
        env_file, key, val = _write_file(ef)
        assert load_file(env_file) == {key: val}
        os.remove(env_file)
    _move(back=True)


def test_system():
    env_vars = load_system()
    # test som of the most common ones
    for key in ['PWD', 'HOME', 'PATH']:
        assert key in env_vars


def test_system_with_file():
    _move()
    env_file, key, val = _write_file('{}.env'.format(TEST_PRE))
    env_vars = load_system()
    env_vars.update(load_file(env_file))
    assert env_vars.get(key) == val
    # test som of the most common ones
    for k in ['PWD', 'HOME', 'PATH']:
        assert k in env_vars
    os.remove(env_file)
    _move(back=True)


def _write_file(fname):
    env_file = os.path.join(CWD, fname)
    key = 'TEST_LOAD'
    val = 'dotenv'
    with open(env_file, 'w') as f:
        if fname.endswith(('.yml', '.yaml')):
            val = 'yaml'
            f.write('{0}: {1}\n'.format(key, val))
        elif fname.endswith(('.json', '.js')):
            val = 'json'
            json.dump({key: val}, f)
        else:
            f.write('{0}={1}\n'.format(key, val))
    return env_file, key, val
